/// <summary>
/// csdic.cs
/// 
/// C# Desktop Integration Components. A set of classes based on
/// really cool and neat early reliase (v 0.8) Java Desktop Integration 
/// Components API (which is cross platform) for C# integration 
/// with MS Windows desktop.
///
/// At present only provides classes for associating C# / .Net 
/// applications with file types and accessing a registered file 
/// type association.
///
/// </summary>
/// <remarks>
///      Created by  : V.Ganesh
///      Created on  : 14th June 2004
///      Version     : 0.1
/// </remarks>
namespace CSDIC {
  namespace FileTypes {

    using System;
    using System.Collections;

    using Microsoft.Win32;

    /// <summary>
    /// This class represents a particular action that can be associated
    /// with a file type
    /// </summary>
    public class Action {
       private string verb, command, description;

       public Action(string verb, string command) : 
              this(verb, command, "") {          
       }
 
       public Action(string verb, string command, string desc) {
          this.verb        = verb;
          this.command     = command;
          this.description = desc;
       }

       public string Verb {
           get { return verb; }
           set { verb = value; }
       }

       public string Command {
           get { return command; }
           set { command = value; }
       }

       public string Description {
           get { return description; }
           set { description = value; }
       }
    } // end of class Action

    /// <summary>
    /// Specifies a file type association, to an action object.
    /// Each file extension is mapped to all the actions represented 
    /// by this object.
    /// </summary>    
    public class Association {

        private ArrayList actions;
        private ArrayList fileExtensions;

        public Association() { 
           actions = new ArrayList();
           fileExtensions = new ArrayList();
        }

        public ArrayList Actions {
           get { return actions; }
           set { actions = value; }
        }

        public ArrayList FileExtensions {
           get { return fileExtensions; }
           set { fileExtensions = value; }
        }
    } // end of class Association
 
    /// <summary>
    /// This class includes methods to access, register, unregister
    /// file type associations from Windows registery.
    /// </summary>
    public class ActionService {

        public ActionService() { }

        public void registerSystemAssociation(Association assoc) {
           registerAssociation(assoc, RegistryLevel.SystemLevel);
        }

        public void unregisterSystemAssociation(Association assoc) {
        }

        public void unregisterUserAssociation(Association assoc) {
        }

        public void registerUserAssociation(Association assoc) {
           registerAssociation(assoc, RegistryLevel.UserLevel);
        }

        public Association getFileExtensionAssociation(string fileExt) {
           // TODO:
           return null;
        }

        protected void registerAssociation(Association assoc, 
                                           RegistryLevel regLevel) {
           string osName = Environment.OSVersion.ToString().ToLower();
           bool isOldWindows = false;
           
           if ((osName.IndexOf("98") != -1) || 
               (osName.IndexOf("me") != -1) || 
               (osName.IndexOf("nt") != -1)) {
               isOldWindows = true;
           } // end if

           if (isOldWindows) {
               regLevel = RegistryLevel.SystemLevel;
           } // end if

           // first register file extensions 
           foreach (string fileExt in assoc.FileExtensions) {
               addFileExt(fileExt, regLevel);
               setActionListByFileExt(assoc.Actions, fileExt, regLevel);
           } // end for           
        }

        protected void addFileExt(string fileExt, RegistryLevel regLevel) {
           RegistryKey fileExtKey = Registry.LocalMachine.OpenSubKey(
                                      @"SOFTWARE\Classes", true);
           if (fileExtKey != null) {
              fileExtKey.CreateSubKey(fileExt);
           } // end if

           fileExtKey.Close();
        }

        protected void setActionListByFileExt(ArrayList actionList, 
                                              string fileExt, 
                                              RegistryLevel regLevel) {
           RegistryKey fileExtKey = Registry.LocalMachine.OpenSubKey(
                              @"SOFTWARE\Classes\" + fileExt, true);

           if (fileExtKey != null) {
              RegistryKey shellKey = fileExtKey.CreateSubKey("shell");
              
              foreach (Action action in actionList) {
                 RegistryKey commandKey = shellKey.CreateSubKey(
                                                      action.Verb);

                 RegistryKey theCommand = commandKey.CreateSubKey("command");
                 theCommand.SetValue("", "\"" + action.Command + "\" \"%1\"");
                 theCommand.Close();
                 commandKey.Close();
              } // end for

              shellKey.Close();

              fileExtKey.Close();
           } // end if        
        }
    } // end of class ActionService

    /// <summary>
    /// The level of registeration.
    /// </summary>
    public enum RegistryLevel { SystemLevel, UserLevel };
  } // end of namespace FileTypes
} // end namespace CSDIC